@@ -765,6 +765,18 @@ def recursive_encode(data, encoding='utf-8'):
             node[key] = value.encode(encoding)
 
 
+def prune_keys(conf, keys={}):
+    '''
+    Returns a deep copy of a configuration removing specified keys.
+    ``prune_keys(conf, {'comment'})`` drops the "comment" key from any dict or sub-dict.
+    '''
+    if isinstance(conf, dict):
+        conf = AttrDict({k: prune_keys(v, keys) for k, v in conf.items() if k not in keys})
+    elif isinstance(conf, (list, tuple)):
+        conf = [prune_keys(v, keys) for v in conf]
+    return conf
+
+
 class TimedRotatingCSVHandler(logging.handlers.TimedRotatingFileHandler):
     '''
     Same as logging.handlers.TimedRotatingFileHandler, but writes to a CSV.
