@@ -5,16 +5,21 @@ The main loop is in :py:func:`pptgen`, which calls other functions as required.
 '''
 
 import copy
+import gramex
 import gramex.data
+import os
 import pandas as pd
 import pptx
+import sys
 from fnmatch import fnmatchcase
+from gramex.config import app_log
 from gramex.transforms import build_transform
 from orderedattrdict import AttrDict
 from pptx import Presentation
 from pptx.oxml import parse_xml
 from pptx.oxml.ns import _nsmap, qn
 from pptx.shapes.shapetree import SlideShapes
+from textwrap import dedent
 from typing import Union, List, Dict
 from . import commands
 
@@ -45,6 +50,7 @@ def pptgen(source: Union[str, pptx.presentation.Presentation],
     :arg handler: if PPTXHandler passes a handler, make it available to the commands as a variable
     :return: target PPTX
     '''
+    # TODO: source can be an expression. PPTXHandler may need to use multiple themes
     prs = source if isinstance(source, pptx.presentation.Presentation) else Presentation(source)
     # Load data with additional variables:
     #   prs: source presentation
@@ -66,12 +72,11 @@ def pptgen(source: Union[str, pptx.presentation.Presentation],
 
     # Loop through each rule (copying them to protect from modification)
     for rule in copy.deepcopy(rules):
-        # slide_data is the data for this rule. If `data:` is specified, override original data
-        slide_data = load_data(rule.get('data', {}), _default_key='function', **data)
-        slides_in_rule = tuple(slide_filter(
-            slides, rule.get('slide-number', []), rule.get('slide-title', [])))
+        slides_in_rule = tuple(slide_filter(slides, rule, data))
+        # If no slides matched, warn the user
         if len(slides_in_rule) == 0:
-            # TODO: warn user that no slides matched this rule
+            app_log.warn('pptgen2: No slide with slide-number: %s, slide-title: %s',
+                         rule.get('slide-number'), rule.get('slide-title'))
             continue
         # Copy slides after the last mapped position of the last slide in this rule
         max_index = max(index for index, slide in slides_in_rule)
@@ -79,7 +84,7 @@ def pptgen(source: Union[str, pptx.presentation.Presentation],
         # Copy all slides into the `copies` list BEFORE applying any rules. Ensures that rules
         # applied to slide 1 don't propagate into 2, 3, etc.
         copies = []
-        copy_seq = iterate_on(rule.get('copy-slide', [None]), slide_data)
+        copy_seq = iterate_on(rule.get('copy-slide', [None]), data)
         for i, (copy_key, copy_val) in enumerate(copy_seq):
             copy_row = AttrDict(pos=i, key=copy_key, val=copy_val, slides=[])
             copies.append(copy_row)
@@ -91,7 +96,9 @@ def pptgen(source: Union[str, pptx.presentation.Presentation],
                 copy_row.slides.append(slide)
         # Apply rules on all copied slides
         for copy_row in copies:
-            slide_data.update(copy=copy_row)        # Rule can use 'copy' as a variable
+            # Include rule-level `data:`. Add copy, slide as variables
+            slide_data = load_data(
+                rule.get('data', {}), _default_key='function', copy=copy_row, **data)
             for slide in copy_row.slides:
                 slide_data['slide'] = slide         # Rule can use 'slide' as a variable
                 transition(slide, rule.get('transition', None), data)
@@ -101,6 +108,21 @@ def pptgen(source: Union[str, pptx.presentation.Presentation],
     return prs
 
 
+# List of commands that can be used in a rule. Shapes cannot have these names
+rule_cmdlist = {
+    'copy-slide',
+    'data',
+    'slide-number',
+    'slide-title',
+    'transition',
+}
+# List of special commands that can be used that are not in commands.cmdlist
+special_cmdlist = {
+    'clone-shape',
+    'data',
+}
+
+
 def apply_commands(rule: Dict[str, dict], shapes, data: dict):
     '''
     Apply commands in rule to change shapes using data.
@@ -111,17 +133,19 @@ def apply_commands(rule: Dict[str, dict], shapes, data: dict):
     :arg dict data: data context for the commands in the rule
     '''
     # Apply every rule to every pattern -- as long as the rule key matches the shape name
-    for shape in shapes:
-        for pattern, spec in rule.items():
+    for pattern, spec in rule.items():
+        if pattern in rule_cmdlist:
+            continue
+        shape_matched = False
+        for shape in shapes:
             if not fnmatchcase(shape.name, pattern):
                 continue
-            shape_data = load_data(spec.get('data', {}), _default_key='function', **data)
-            shape_data['shape'] = shape         # Commands can use 'shape' as a variable
+            shape_matched = True
             # Clone all slides into the `clones` list BEFORE applying any command. Ensures that
             # commands applied to the shape don't propagate into its clones
             clones = []
-            clone_seq = iterate_on(spec.get('clone-shape', [None]), shape_data)
-            parent_clone = shape_data.get('clone', None)
+            clone_seq = iterate_on(spec.get('clone-shape', [None]), data)
+            parent_clone = data.get('clone', None)
             for i, (clone_key, clone_val) in enumerate(clone_seq):
                 if i > 0:
                     # This copies only a shape, group or image. Not table, chart, media, equation,
@@ -132,14 +156,25 @@ def apply_commands(rule: Dict[str, dict], shapes, data: dict):
                 clones.append(AttrDict(pos=i, key=clone_key, val=clone_val, shape=shape,
                                        parent=parent_clone))
             # Run commands in the spec on all cloned shapes
-            for clone in clones:
-                shape_data.update(clone=clone)
+            is_group = shape.element.tag.endswith('}grpSp')
+            for i, clone in enumerate(clones):
+                # Include shape-level `data:`. Add shape, clone as variables
+                shape_data = load_data(
+                    spec.get('data', {}), _default_key='function', shape=shape, clone=clone,
+                    **{k: v for k, v in data.items() if k not in {'shape', 'clone'}})
                 for cmd in spec:
                     if cmd in commands.cmdlist:
                         commands.cmdlist[cmd](clone.shape, spec[cmd], shape_data)
+                    # Warn on unknown commands. But don't warn on groups -- they have sub-shapes
+                    elif cmd not in special_cmdlist and not is_group:
+                        app_log.warn('pptgen2: Unknown command: %s on shape: %s', cmd, pattern)
                 # If the shape is a group, apply spec to each sub-shape
-                if shape.element.tag.endswith('}grpSp'):
+                if is_group:
                     apply_commands(spec, SlideShapes(clone.shape.element, shapes), shape_data)
+        # Warn if the pattern is neither a shape nor a command
+        if (not shape_matched and pattern not in special_cmdlist and
+                pattern not in commands.cmdlist):
+            app_log.warn('pptgen2: No shape matches pattern: %s', pattern)
 
 
 def load_data(_conf, _default_key: str = None, **kwargs) -> dict:
@@ -241,25 +276,27 @@ def pick_only_slides(prs: Presentation, only: Union[int, List[int]] = None) -> l
     return list(prs.slides)
 
 
-def slide_filter(slides, numbers: Union[int, List[int]], titles: Union[str, List[str]]):
+def slide_filter(slides, rule: dict, data: dict):
     '''
     Filter slides. Return iterable of (index, slide) for only those slides matching numbers/titles.
 
     :arg Slides slides: a Slides object (e.g. ``prs.slides``) to select slides from
-    :arg int/list numbers: slide number(s) to filter. 1 is the first slide. [1, 3] is slides 1 & 3
-    :arg str/list titles: slide title pattern(s) to filter. "*Match*" matches all slides with
-        "match" anywhere in the title (case-insensitive)
+    :arg dict rule: a dict that may have a ``slide-number`` or ``slide-title`` key to filter by.
+        These can be expressions. ``slide-number`` is the slide number(s) to filter. 1 is the first
+        slide. ``[1, 3]`` is slides 1 & 3. ``slide-title`` has the slide title pattern(s) to filter
+        e.g. ``*Match*`` matches all slides with "match" anywhere in the title (case-insensitive).
+    :arg dict data: data context for the rule
     :return: an iterable that yields (index, slide)
     '''
-    # TODO: allow slide numbers and slide-titles to be data-driven
-    # slide_numbers is the SET of 1-indexed slide numbers this rule applies to
-    slide_numbers = set(numbers if isinstance(numbers, list) else [numbers])
-
+    numbers = commands.expr(rule.get('slide-number', []), data)
+    titles = commands.expr(rule.get('slide-title', []), data)
+    # numbers is the SET of 1-indexed slide numbers this rule applies to
+    numbers = set(numbers if isinstance(numbers, list) else [numbers])
     # titles is a LIST of title patterns - ANY of which the slide title must match
     titles = titles if isinstance(titles, list) else [titles]
-
+    # yield only the slides that match these conditions
     for index, slide in enumerate(slides):
-        if slide_numbers and (index + 1 not in slide_numbers):
+        if numbers and (index + 1 not in numbers):
             continue
         title = slide.shapes.title.text if slide.shapes.title else ''
         if titles and (not any(fnmatchcase(title.lower(), pattern.lower()) for pattern in titles)):
@@ -295,14 +332,17 @@ def copy_slide(prs, source, target_index):
             # This removes notes slide formatting and just copies the text
             dest.notes_slide.notes_text_frame.text = source.notes_slide.notes_text_frame.text
             continue
-        # TODO: https://github.com/scanny/python-pptx/issues/132#issuecomment-414001942
         elif 'chart' in val.reltype:
-            pass
-            # partname = target.package.next_partname(pptx.parts.chart.ChartPart.partname_template)
-            # target = pptx.parts.chart.ChartPart(partname, target.content_type,
-            #     copy.deepcopy(target._element), package=target.package)
-            # target.chart_workbook.xlsx_part = pptx.parts.chart.EmbeddedXlsxPart.new(
-            #     target.chart_workbook.xlsx_part.blob, target.package)
+            # https://github.com/scanny/python-pptx/issues/132#issuecomment-414001942
+            # Does not work for Treemap, modern charts.
+            # TODO: Replace with underlying lxml code
+            partname = target.package.next_partname(pptx.parts.chart.ChartPart.partname_template)
+            xlsx_blob = target.chart_workbook.xlsx_part.blob
+            target = pptx.parts.chart.ChartPart(
+                partname, target.content_type, copy.deepcopy(target._element),
+                package=target.package)
+            target.chart_workbook.xlsx_part = pptx.parts.chart.EmbeddedXlsxPart.new(
+                xlsx_blob, target.package)
         # TODO: handle diagrams
         dest.part.rels.add_relationship(val.reltype, target, val.rId, val.is_external)
     # Move appended slide into target_index
@@ -395,3 +435,49 @@ def iterate_on(spec, data: dict):
         return val
     else:
         raise ValueError('Cannot iterate over %s: %r' % (type(val), val))
+
+
+def commandline(args=None):
+    '''
+    usage: slidesense [config.yaml] [url-name] [--source=...] [--target=...] [--data=...]
+
+    Generates target PPTX from a source PPTX, applying rules in config file and opens it.
+    If no config file is specified, uses `gramex.yaml` in the current directory.
+
+    The config file can have a pptgen configuration like {source: ..., target: ..., rules: ...}
+    or be a `gramex.yaml` with `url: {url-name: {handler: PPTXHandler, kwargs: {source: ...}}}`
+    Rules are picked up from the first PPTXHandler URL that matches `url-name`,
+    or the first PPTXHandler in `gramex.yaml`.
+
+    --source=... overrides source PPTX path in config file
+    --target=... overrides target PPTX path in config file (defaults to output.pptx)
+    --data=...   overrides data file in config path
+    --no-open    don't open target PPTX after generating it
+    '''
+    args = gramex.parse_command_line(sys.argv[1:] if args is None else args)
+
+    if 'help' in args or (not args['_'] and not os.path.exists('gramex.yaml')):
+        return gramex.console(dedent(commandline.__doc__).strip())
+
+    config_file, *urls = args.pop('_') or ['gramex.yaml']
+    conf = gramex.cache.open(config_file, 'config')
+
+    if 'url' in conf:
+        for key, spec in conf.url.items():
+            if spec.handler == 'PPTXHandler':
+                if not urls or any(url in key for url in urls):
+                    rules = spec.kwargs
+                    break
+        else:
+            return app_log.error(f'No PPTXHandler matched in file: {config_file}')
+    elif any(key in conf for key in ('source', 'target', 'data', 'rules')):
+        rules = conf
+    else:
+        return app_log.error(f'No rules found in file: {config_file}')
+
+    gramex.config.merge(rules, args)
+    rules.setdefault('target', 'output.pptx')
+    rules.setdefault('mode', 'expr')
+    gramex.pptgen2.pptgen(**rules)
+    if not rules.get('no-open', False):
+        os.startfile(rules['target'])
